module DiffinDiffsBase

using Base: @propagate_inbounds
using CSV
using CodecZlib: GzipDecompressorStream
using DataAPI
using DataAPI: refarray, refpool, invrefpool
using Dates: Period, TimeType
using LinearAlgebra: Diagonal
using Missings: allowmissing, disallowmissing
using PooledArrays: _label
using PrettyTables: pretty_table
using Reexport
using StatsAPI: stderror
using StatsBase: CoefTable, Weights, uweights
using StatsFuns: normccdf, norminvcdf, tdistccdf, tdistinvcdf
@reexport using StatsModels
using StatsModels: Schema
@reexport using StatsProcedures
using StatsProcedures: ≊, _args_kwargs, _parse!
using StructArrays: StructArray
using Tables
using Tables: AbstractColumns, istable, columnnames, getcolumn

import Base: ==, +, -, *, isless, show, parent, view, diff
import Base: eltype, firstindex, lastindex, getindex, iterate, length
import Missings: allowmissing, disallowmissing
import StatsAPI: coef, vcov, confint, nobs, dof_residual, responsename, coefnames, weights,
    coeftable
import StatsModels: concrete_term, schema, termvars, lag, lead
import StatsProcedures: required, default, transformed, combinedargs, copyargs, _show_args

# Reexport objects from StatsAPI and StatsBase
export coef, vcov, stderror, confint, nobs, dof_residual, responsename, coefnames, weights,
    coeftable

export cb,
       ≊,
       exampledata,

       RotatingTimeValue,
       rotatingtime,
       RotatingTimeArray,

       VecColumnTable,
       VecColsRow,
       subcolumns,
       apply,
       apply_and!,
       apply_and,
       TableIndexedMatrix,

       ScaledArray,
       ScaledVector,
       ScaledMatrix,
       scale,
       align,

       TreatmentSharpness,
       SharpDesign,
       sharp,
       AbstractTreatment,
       DynamicTreatment,
       dynamic,

       ParallelCondition,
       Unconditional,
       unconditional,
       CovariateConditional,
       ParallelStrength,
       Exact,
       exact,
       Approximate,
       AbstractParallel,
       UnspecifiedParallel,
       unspecifiedpr,
       TrendParallel,
       TrendOrUnspecifiedPR,
       NeverTreatedParallel,
       nevertreated,
       NotYetTreatedParallel,
       notyettreated,
       istreated,
       istreated!,

       TermSet,
       termset,
       eachterm,
       TreatmentTerm,
       treat,

       findcell,
       cellrows,
       settime,
       aligntime,
       PanelStructure,
       setpanel,
       findlag!,
       findlead!,
       ilag!,
       ilead!,
       diff!,
       diff,

       CheckData,
       GroupTreatintterms,
       GroupXterms,
       GroupContrasts,
       CheckVars,
       GroupSample,
       MakeWeights,

       DiffinDiffsEstimator,
       DefaultDID,
       did,
       didspec,
       @did,
       AbstractDIDResult,
       DIDResult,
       AggregatedDIDResult,
       vce,
       treatment,
       outcomename,
       treatnames,
       treatcells,
       ntreatcoef,
       treatcoef,
       treatvcov,
       coefinds,
       ncovariate,
       agg,
       SubDIDResult,
       TransformedDIDResult,
       TransSubDIDResult,
       lincom,
       rescale,
       ExportFormat,
       StataPostHDF,
       getexportformat,
       setexportformat!,
       post!

include("tables.jl")
include("utils.jl")
include("time.jl")
include("ScaledArrays.jl")
include("treatments.jl")
include("parallels.jl")
include("terms.jl")
include("operations.jl")
include("procedures.jl")
include("did.jl")

end
